package net.vidageek.mirror.reflect;

import java.lang.annotation.Annotation;

/**
 * @author dnfeitosa
 * 
 */
public class AnnotationHandler<T extends Annotation> {

	private final Class<?> clazz;
	private final Class<T> annotation;

	public AnnotationHandler(final Class<?> clazz, final Class<T> annotation) {
		if (clazz == null) {
			throw new IllegalArgumentException("Argument clazz cannot be null.");
		}
		if (annotation == null) {
			throw new IllegalArgumentException(
					"Argument annotation cannot be null.");
		}
		this.clazz = clazz;
		this.annotation = annotation;
	}

	public T atField(final String fieldName) {
		if (fieldName == null || fieldName.trim().length() == 0) {
			throw new IllegalArgumentException(
					"fieldName cannot be null or empty.");
		}
		return new FieldReflector(fieldName).onClass(clazz).getAnnotation(
				annotation);
	}

	@SuppressWarnings("unchecked")
	public MethodAnnotationHandler<T> atMethod(final String methodName) {
		return new MethodAnnotationHandler(clazz, methodName, annotation);
	}

	public T atClass() {
		return clazz.getAnnotation(annotation);
	}

}
